#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          25-06-2018
//      Created for Project :   FLAME
"""

import pytest
from sqlalchemy import create_engine
from sqlalchemy.exc import ProgrammingError, OperationalError
from clmcservice.models.meta import DBSession, Base


def create_test_database(db_name):
    """
    This function creates a test database with the given name. If the database already exists, it is recreated.

    :param db_name: the test database name
    """

    engine = create_engine("postgresql://clmc:clmc_service@localhost:5432/postgres", echo=False)
    conn = engine.connect().execution_options(autocommit=False)
    conn.execute("ROLLBACK")  # connection is already in a transaction, hence roll back (postgres databases cannot be created in a transaction)
    try:
        conn.execute("DROP DATABASE %s" % db_name)
        print("\nOld database '{0}' has been deleted.".format(db_name))
    except ProgrammingError:
        # database probably doesn't exist
        conn.execute("ROLLBACK")
    except OperationalError as e:
        print(e)
        # database exists and is probably being used by other users
        conn.execute("ROLLBACK")
        conn.close()
        engine.dispose()
        raise pytest.exit("Old test database cannot be deleted.")

    conn.execute("CREATE DATABASE %s" % db_name)
    conn.close()
    engine.dispose()
    print("\nNew test database '{0}' has been created.".format(db_name))


def initialise_database(db_name):
    """
    This function initialises the test database by binding the shared DB session to a new connection engine and creating tables for all models.

    :param db_name: test database name
    :return: the configured DB session, which is connected to the test database
    """

    engine = create_engine('postgresql://clmc:clmc_service@localhost:5432/{0}'.format(db_name))  # create an engine to connect to the test database
    DBSession.configure(bind=engine)  # configure the database session
    Base.metadata.bind = engine
    Base.metadata.create_all()  # create tables for all models

    return DBSession, engine


def drop_test_database(db_name):
    """
    This function removes the test database with the given name, if it exists

    :param db_name: the test database name
    """

    engine = create_engine("postgresql://clmc:clmc_service@localhost:5432/postgres", echo=False)
    conn = engine.connect().execution_options(autocommit=False)
    conn.execute("ROLLBACK")  # connection is already in a transaction, hence roll back (postgres databases cannot be created in a transaction)
    try:
        conn.execute("DROP DATABASE %s" % db_name)
        print("\nTest database '{0}' has been deleted.".format(db_name))
    except ProgrammingError:
        # database probably doesn't exist
        conn.execute("ROLLBACK")
    except OperationalError as e:
        print(e)
        # database is probably being used by other users
        conn.execute("ROLLBACK")

    conn.close()
    engine.dispose()


@pytest.fixture(scope='module', autouse=True)
def testing_db_session():

    test_database = "whoamitestdb"
    create_test_database(test_database)  # create a database used for executing the unit tests
    db_session, engine = initialise_database(test_database)  # initialise the database with the models and retrieve a db session

    yield db_session  # return the db session if needed in any of the tests

    db_session.remove()  # remove the db session
    engine.dispose()  # dispose from the engine
    drop_test_database(test_database)  # remove the test database
