#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road, 
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Michael Boniface
##      Created Date :          02-03-2018
##      Updated By :            Nikolay Stanchev
##      Updated Date :          30-08-2018
##      Created for Project :   FLAME
"""

import pytest


@pytest.mark.parametrize("measurement, query, expected_result", [
    ('nginx', 'SELECT mean("requests") AS "mean" FROM "MS_Template_1"."autogen"."nginx"', 0),
    ('cpu', 'SELECT mean("usage_idle") AS "mean" FROM "MS_Template_1"."autogen"."cpu"', 0),
    ('mongodb', 'SELECT mean("net_in_bytes") AS "mean" FROM "MS_Template_1"."autogen"."mongodb"', 0),
    ('net', 'SELECT mean("bytes_sent") AS "mean" FROM "MS_Template_1"."autogen"."net"', 0),
    ('disk', 'SELECT mean("free") AS "mean" FROM "MS_Template_1"."autogen"."disk"', 0),
    ('mem', 'SELECT mean("free") AS "mean" FROM "MS_Template_1"."autogen"."mem"', 0),
    ('service_config_state', 'SELECT mean("loaded.active.running_count") AS "mean" FROM "MS_Template_1"."autogen"."service_config_state" WHERE "resource"=\'nginx.service\'', 0),
    # Report MINIO's HTTP request response time (as a rolling difference of the sum total)
    ('minio_http_requests_duration_seconds', 'SELECT difference(max("sum")) AS "mean" FROM "MS_Template_1"."autogen"."minio_http_requests_duration_seconds" WHERE time > now() - 1h GROUP BY time(10s)',0),
    # Report the average change in difference of MINIO's HTTP response time (the inner query determines a rolling difference between sampling periods [respTimeDiff])
    ('minio_http_requests_duration_seconds', 'SELECT mean("respTimeDiff") AS "mean" FROM (SELECT difference(max("sum")) AS "respTimeDiff" FROM "MS_Template_1"."autogen"."minio_http_requests_duration_seconds" WHERE time > now() - 1h GROUP BY time(10s))',0)
    ])
def test_all_inputs(influxdb, measurement, query, expected_result):
    """
    Tests measurements are received from an input plugin aggregated across all services

    :param influxdb: the influx db client fixture
    :param measurement: the measurement to test
    :param query: the query to execute
    :param expected_result: the expected result from the query
    """

    query_result = influxdb.query('SHOW measurements ON "MS_Template_1"')
    points = list(query_result.get_points())
    assert any(p['name'] == measurement for p in points), "{0} not in measurement list".format(measurement)
    
    query_result = influxdb.query(query)
    points = next(query_result.get_points())  # get_points() returns a generator, to take the first element we can use the next() function
    actual_result = points['mean']
    assert actual_result > expected_result, "actual result {0} is not > expected result {1} for query {2}".format(actual_result, str(expected_result), query)


@pytest.mark.parametrize("query", [
    'SELECT * FROM "MS_Template_1"."autogen"."nginx" GROUP BY *',
    'SELECT * FROM "MS_Template_1"."autogen"."cpu" GROUP BY *',
    'SELECT * FROM "MS_Template_1"."autogen"."mongodb" GROUP BY *',
    'SELECT * FROM "MS_Template_1"."autogen"."net" GROUP BY *',
    'SELECT * FROM "MS_Template_1"."autogen"."disk" GROUP BY *',
    'SELECT * FROM "MS_Template_1"."autogen"."mem" GROUP BY *',
    'SELECT * FROM "MS_Template_1"."autogen"."service_config_state" WHERE "resource"=\'nginx.service\' GROUP BY *',
    'SELECT * FROM "MS_Template_1"."autogen"."minio_http_requests_duration_seconds" GROUP BY *'
])
def test_global_tag_filtering(influxdb, query):
    """
    Tests that the global tags are inserted correctly into the global configuration using the install CLMC agent script

    :param influxdb: the influx db client fixture
    :param query: the query to execute
    """

    query_result = influxdb.query(query).items()[0]
    tags = query_result[0][1].keys()

    assert set(tags).issuperset({"flame_sfc", "flame_sfci", "flame_sfp", "flame_sf", "flame_sfe", "flame_server", "flame_location"})
